BASE_URL = r'https://localhost:44320/Operations'
DEFAULT_ALGORITHM = 'insertion_beam'
DEFAULT_OBJECTIVE = 'cmax'
DEFAULT_BEAM_WIDTH = 5

PRINT_RESPONSES = True
PRINT_RESPONES_DEBUG_TIMES = False
PRINT_METHOD_TIMES = False
PRINT_SCHEDULE_CORRECTION_MESSAGE = False
PRINT_DEBUG_MESSSAGES = False

SHOW_CONFLICT_GRAPH = False
SHOW_RESULT_SCHEDULE_GRAPH = False
SHOW_GANTT = False
